# -*- coding: utf-8 -*-
#
# This file is part of the enjoy project
#
# Copyright (c) 2021 Tiago Coutinho
# Distributed under the GPLv3 license. See LICENSE for more info.

"""Top-level package for Enjoy."""

__author__ = "Jose Tiago Macara Coutinho"
__email__ = "coutinhotiago@gmail.com"
__version__ = "0.1.2"
